{% if season %}
    {% include 'driver27/season/menu-pre.html' %}
{% elif competition %}
    {% include 'driver27/competition/menu-pre.html' %}
{% else %}
    {% include 'driver27/global/menu-pre.html' %}
{% endif %}