{% for record_code, record_label in record_codes %}
<option value="{{ record_code }}"
{% if record_code == record %}selected{% endif %}
{% if rank_opt|lower == 'doubles' and record_code not in doubles_record_codes %}
    style="background-color: lightsalmon"
{% endif %}
>{{ record_label }}</option>
{% endfor %}
