<table class="table">
    <thead>
        <tr>
            <th>Team</th>
            <th>Country</th>
        </tr>
    </thead>
    <tbody>

        {% for team in teams %}
        <tr>
            <td>{{ team.name }}</td>
            <td>{{ team.country }}</td>
        </tr>
            {% if ok and can_save %}
                <input type="hidden" name="items" value="{{ team.pk }}">
            {% endif %}
        {% endfor %}
    </tbody>

</table>