{% load i18n %}
<li class="nav-item">
        <a class="nav-link" href="{% url 'driver27:competition:driver' competition.slug %}">{%  trans 'Drivers' %}</a>
    </li>

<li class="nav-item">
        <a class="nav-link" href="{% url 'driver27:competition:team' competition.slug %}">{%  trans 'Teams' %}</a>
    </li>