{% extends 'driver27/base.html' %}
{% load i18n %}
{% block dr27_menu %}

    {% include 'driver27/_selector_menu.html' %}
    {% include 'driver27/driver/menu.html' %}
{% endblock %}
{% block dr27_content %}
    {{ block.super }}
    {% load champion_filter i18n %}
    <form method="POST" class="form-inline">
        {% csrf_token %}
        <div class="form-group col-md-4">
            {% if competition %}
                <input type="hidden" name="competition" value="{{ competition.slug }}"/>
            {% endif %}
            {% if season %}
                <input type="hidden" name="year" value="{{ season.year }}"/>
            {% endif %}
            <label for="record" class="control-label">Record:</label>
            <select id="record" name="record" class="form-control input-sm">
                {% if not record %}
                    <option value="">---</option>
                {% endif %}
                {% include 'driver27/driver/record-optgroup.html' %}
            </select>
        </div>
        <div class="col-md-4 form-group">
            <label for="rank_opt" class="control-label">{% trans 'Filter' %}</label>
            <select id="rank_opt" name="rank_opt" class="form-control input-sm">

                {% include 'driver27/driver/streak-opt.html' %}
            </select>
        </div>
    </form>


    <div id="driver-stats" class="stats" data-standing-model="driver"></div>
{% endblock %}

{% block bootstrap3_extra_script %}
    {{ block.super }}
    {% include 'driver27/include_rank_js.html' %}
{% endblock %}