{% extends 'driver27/base.html' %}
{% load bootstrap3 i18n %}

{% block dr27_menu %}
    {{ block.super }}
{% endblock %}



{% block dr27_content %}
    {{ block.super }}
    <table class="table table-condensed table-striped table-hover">
        <tr>
            <th>{% trans 'Competition' %}</th>
            <th>{% trans 'Seasons' %}</th>
        </tr>
{#        {% include 'driver27/competition/competition-season-list.html' %}#}
    </table>


{% endblock %}