{% load i18n %}
    <li class="nav-item">
        <a>>></a>
    </li>
    <li class="nav-item">
        <a class="nav-link" href="{% url 'driver27:season:driver' season.competition.slug season.year %}">{%  trans 'Drivers' %}</a>
    </li>
    <li class="nav-item">
        <a class="nav-link" href="{% url 'driver27:season:team' season.competition.slug season.year %}">{%  trans 'Teams' %}</a>
    </li>
    <li class="nav-item">
        <a class="nav-link" href="{{ season.get_races_url }}">{%  trans 'Races' %}</a>
    </li>