{% load i18n %}
<option value="">
    ---
</option>
<option value="streak" {% if rank_opt|lower == 'streak' %}selected{% endif %}>
    {% trans 'Current streak.' %}
</option>
<option value="streak_top" {% if rank_opt|lower == 'streak_top' %}selected{% endif %}>
    {% trans 'Best streak.' %}
</option>
<option value="stats" {% if rank_opt|lower == 'stats' %}selected{% endif %}>
    {% trans 'Record by drivers of team.' %}
</option>
<option value="races" {% if rank_opt|lower == 'races' %}selected{% endif %}>
    {% trans 'Records by race with at least one driver.' %}
</option>
<option value="doubles" {% if rank_opt|lower == 'doubles' %}selected{% endif %}>
    {% trans 'Records by race with at least two drivers.' %}
</option>
{% if not season %}
<option value="seasons" {% if rank_opt|lower == 'seasons' %}selected{% endif %}>
    {% trans 'Record by season' %}
</option>
{% endif %}